#include "delay.h"
#include "msp.h"
#include <stdint.h>

int delay_cycles(uint32_t counter)
{
    TIMER32_1->CONTROL = TIMER32_CONTROL_SIZE |
        TIMER32_CONTROL_ONESHOT;

    TIMER32_1->LOAD = counter;

    __enable_irq();
    NVIC->ISER[0] = 1 << ((T32_INT1_IRQn) & 31);

    TIMER32_1->CONTROL |= TIMER32_CONTROL_ENABLE |
        TIMER32_CONTROL_IE;

    // Disable sleep on exit from ISR
    SCB->SCR &= ~SCB_SCR_SLEEPONEXIT_Msk;

    // Enter LPM0 and wait for the timer interrupt to wake-up
    __sleep();
    __no_operation();

    return 0;
}


int delay_ms(uint16_t ms)
{
    int32_t freq;
    uint32_t counter;

    if((freq = get_DCO()) == -1)
        return -1;

    // Calculate clock cycles based off frequency
    counter = ms * (freq / MILLI);

    return delay_cycles(counter);
}

int delay_us(uint16_t us)
{
    int32_t freq;
    uint32_t counter;

    if((freq = get_DCO()) == -1)
        return -1;

    // Calculate clock cycles based off frequency
    counter = us * (freq / MICRO);

    // get rid of overhead
    counter -= 20 * (freq / MICRO) + 180;

    return delay_cycles(counter);
}

int set_DCO(uint32_t freq)
{
    if(freq == FREQ_48MHZ)
    {
        while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
        PCM->CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_AMR_1;
        while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
        FLCTL->BANK0_RDCTL = (FLCTL->BANK0_RDCTL &
             ~(FLCTL_BANK0_RDCTL_WAIT_MASK)) | FLCTL_BANK0_RDCTL_WAIT_1;
        FLCTL->BANK1_RDCTL  = (FLCTL->BANK0_RDCTL &
             ~(FLCTL_BANK1_RDCTL_WAIT_MASK)) | FLCTL_BANK1_RDCTL_WAIT_1;
    }

    CS->KEY = CS_KEY_VAL;
    CS->CTL0 = 0;
    CS->CTL0 = freq;
    CS->CTL1 = CS_CTL1_SELA_2 | CS_CTL1_SELS_3 | CS_CTL1_SELM_3;
    CS->KEY = 0;

    switch(freq)
    {
        case FREQ_1_5MHZ:
            return _1_5MHZ;
        case FREQ_3MHZ:
            return _3MHZ;
        case FREQ_6MHZ:
            return _6MHZ;
        case FREQ_12MHZ:
            return _12MHZ;
        case FREQ_24MHZ:
            return _24MHZ;
        case FREQ_48MHZ:
            return _48MHZ;
        default:
            return -1;
    }
}

int get_DCO()
{
    switch(CS->CTL0 & (CS_CTL0_DCORSEL0 | CS_CTL0_DCORSEL1 | CS_CTL0_DCORSEL2))
    {
        case FREQ_1_5MHZ:
            return _1_5MHZ;
        case FREQ_3MHZ:
            return _3MHZ;
        case FREQ_6MHZ:
            return _6MHZ;
        case FREQ_12MHZ:
            return _12MHZ;
        case FREQ_24MHZ:
            return _24MHZ;
        case FREQ_48MHZ:
            return _48MHZ;
        default:
            return -1;
    }
}

void T32_INT1_IRQHandler(void)
{
    TIMER32_1->INTCLR |= BIT0;              // Clear Timer32 interrupt flag

    // Disable the timer and interrupt
    TIMER32_1->CONTROL &= ~(TIMER32_CONTROL_ENABLE |
            TIMER32_CONTROL_IE);
}
